//  Copyright 2004 The Apache Software Foundation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package org.apache.hivemind.sdl;

import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

import org.apache.hivemind.Resource;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.xml.sax.Attributes;
import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;
import org.xml.sax.helpers.DefaultHandler;

/**
 * An adaptor that allows the content of an SDL file (as a {@link Resource}) to
 * be parsed into an existing {@link org.w3c.dom.Document}.
 * 
 * @author Howard Lewis Ship
 */
public class SDLDocumentAdaptor extends DefaultHandler
{
    private Document _document;
    private List _nodeStack = new ArrayList();

    public SDLDocumentAdaptor(Document document)
    {
        _document = document;
        _nodeStack.add(document);
    }

    public void parse(Resource resource) throws SAXParseException, IOException
    {
        SDLResourceParser parser = new SDLResourceParser();

        parser.parseResource(resource, this);
    }

    private Node peek()
    {
        return (Node) _nodeStack.get(_nodeStack.size() - 1);
    }

    public void characters(char[] ch, int start, int length) throws SAXException
    {
        Node currentElement = peek();

        String data = new String(ch, start, length);

        Node textNode = _document.createCDATASection(data);

        currentElement.appendChild(textNode);
    }

    public void endElement(String uri, String localName, String qName) throws SAXException
    {
        pop();
    }

    private void pop()
    {
        int count = _nodeStack.size();

        _nodeStack.remove(count - 1);
    }

    public void startElement(String uri, String localName, String qName, Attributes attributes)
        throws SAXException
    {
        Node parentElement = peek();

        // The SDL parser provides just localName, never qName
        Element newElement = _document.createElement(localName);

        int count = attributes.getLength();
        for (int i = 0; i < count; i++)
        {
            String name = attributes.getLocalName(i);
            String value = attributes.getValue(i);

            newElement.setAttribute(name, value);
        }

        parentElement.appendChild(newElement);

        push(newElement);
    }

    private void push(Node node)
    {
        _nodeStack.add(node);
    }
}
